/*
  server.js

  - "Solo" and "Together" arrays:
    - Start with 5 each, index 0..4 are "base" points.
    - Additional "earned" points are appended (index >= 5).
  - Weekly reset => set ALL items to false (keep array lengths).
  - POST /add-points => appends new "false" items to "soloUsed" or "togetherUsed".
  - Admin route: /admin/remove-all-earned => remove appended points beyond index 4 in both arrays.
*/

const express = require('express');
const cors = require('cors');

// ----- CONFIG -----
const PORT = 4000;

// Default base counts
const SOLO_BASE_COUNT = 5;
const TOGETHER_BASE_COUNT = 5;

// Passwords
const POINTS_PASSWORD = 'mySecretPassword'; // for adding points
const ADMIN_PASSWORD = 'adminSecret';       // for admin panel

// Timer durations (ms)
const FIFTEEN_MINUTES = 15 * 60 * 1000;
const TWO_MINUTES = 2 * 60 * 1000;

// ----- IN-MEMORY STATE -----
// Start with 5 false for each
let soloUsed = Array(SOLO_BASE_COUNT).fill(false);
let togetherUsed = Array(TOGETHER_BASE_COUNT).fill(false);

// Logs: e.g. { type:'solo'|'together'|'movie'|'episode', index, usedAt:'ISOString' }
let logs = [];

let lastReset = null;

// Timer
let timerRunning = false;
let timeRemaining = 0;
let isPaused = false;
let activeUsage = null; // e.g. { category:'solo'|'together', index }
let ringing = false;

// -----------------------------
//  Weekly Reset Logic
// -----------------------------
function getMonday(d = new Date()) {
    const day = d.getDay(); // 0=Sun,1=Mon,...
    const diff = d.getDate() - day + (day === 0 ? -6 : 1);
    d.setDate(diff);
    return d.toISOString().split('T')[0];
}

function resetIfNewWeek() {
    const currentMonday = getMonday();
    if (!lastReset || lastReset !== currentMonday) {
        // Keep array lengths, reset all to false
        soloUsed = soloUsed.map(() => false);
        togetherUsed = togetherUsed.map(() => false);

        lastReset = currentMonday;
        console.log('[Server] Weekly reset triggered:', currentMonday);
    }
}

// Decrement the timer
setInterval(() => {
    if (timerRunning && !isPaused && timeRemaining > 0) {
        timeRemaining -= 1000;
        if (timeRemaining <= 0) {
            // If finishing a 15-min => ring; if finishing 2-min => just end
            timerRunning = false;
            timeRemaining = 0;

            // If we aren't already ringing, that means we ended the 15-min block => ring
            if (!ringing) {
                ringing = true;
                console.log('[Server] 15-minute timer ended => ringing=true');
            }
        }
    }
}, 1000);

// ----- EXPRESS -----
const app = express();
app.use(cors());
app.use(express.json());

// Weekly reset check
app.use((req, res, next) => {
    resetIfNewWeek();
    next();
});

// GET /state => entire state
app.get('/state', (req, res) => {
    res.json({
        soloUsed,
        togetherUsed,
        logs,
        lastReset,
        timerRunning,
        timeRemaining,
        isPaused,
        activeUsage,
        ringing,
    });
});

// POST /use-bubble => single bubble => 15-min
app.post('/use-bubble', (req, res) => {
    const { category, index } = req.body;

    if (timerRunning && !ringing && timeRemaining > 0) {
        return res.status(400).json({ error: 'A timer is already active.' });
    }
    // If ringing => auto-ignore
    if (ringing) {
        timerRunning = false;
        timeRemaining = 0;
        isPaused = false;
        activeUsage = null;
        ringing = false;
        console.log('[Server] Timer was ringing => auto-ignored, new usage starts.');
    }

    let arr;
    if (category === 'solo') arr = soloUsed;
    else if (category === 'together') arr = togetherUsed;
    else return res.status(400).json({ error: 'Invalid category' });

    if (index < 0 || index >= arr.length) {
        return res.status(400).json({ error: 'Index out of range' });
    }
    if (arr[index] === true) {
        return res.status(400).json({ error: 'That bubble is already used.' });
    }

    // Mark used
    arr[index] = true;
    const usedAt = new Date().toISOString();
    logs.push({ type: category, index, usedAt });

    // Start 15-min
    timerRunning = true;
    timeRemaining = FIFTEEN_MINUTES;
    isPaused = false;
    activeUsage = { category, index };
    console.log(`[Server] ${category} #${index + 1} => 15-min started`);
    return res.json({ success: true });
});

// POST /check-password => new endpoint to quickly validate POINTS_PASSWORD
app.post('/check-password', (req, res) => {
    const { password } = req.body;
    if (password !== POINTS_PASSWORD) {
        return res.status(403).json({ error: 'Incorrect password.' });
    }
    return res.json({ success: true });
});

// POST /add-points => appends "amount" new false items to soloUsed/togetherUsed
app.post('/add-points', (req, res) => {
    const { password, category, amount } = req.body;
    // We could re-check password if we want.
    // If you rely on /check-password, then skip here or just do it again:
    if (password !== POINTS_PASSWORD) {
        return res.status(403).json({ error: 'Incorrect password.' });
    }
    if (!['solo', 'together'].includes(category)) {
        return res.status(400).json({ error: 'Invalid category' });
    }
    if (typeof amount !== 'number' || amount < 1 || amount > 4) {
        return res.status(400).json({ error: 'Amount must be 1..4' });
    }

    if (category === 'solo') {
        for (let i = 0; i < amount; i++) {
            soloUsed.push(false);
        }
        console.log(`[Server] +${amount} appended to soloUsed => length now ${soloUsed.length}`);
    } else {
        for (let i = 0; i < amount; i++) {
            togetherUsed.push(false);
        }
        console.log(`[Server] +${amount} appended to togetherUsed => length now ${togetherUsed.length}`);
    }
    return res.json({ success: true });
});

// Pause/Resume
app.post('/pause-resume', (req, res) => {
    if (!timerRunning && timeRemaining <= 0) {
        return res.status(400).json({ error: 'No active timer.' });
    }
    if (ringing) {
        return res.status(400).json({ error: 'Timer is ringing, cannot pause.' });
    }
    isPaused = !isPaused;
    console.log('[Server] Timer pause =>', isPaused);
    return res.json({ success: true, isPaused });
});

// Finish-up => only if ringing
app.post('/finish-up', (req, res) => {
    if (!ringing) {
        return res.status(400).json({ error: 'Not currently ringing. 2-min only after 15-min ends.' });
    }
    timerRunning = true;
    timeRemaining = TWO_MINUTES;
    isPaused = false;
    ringing = false;
    console.log('[Server] 2-min finish-up started');
    return res.json({ success: true });
});

// Cancel finish-up
app.post('/cancel-finish-up', (req, res) => {
    if (!timerRunning || ringing || timeRemaining > TWO_MINUTES) {
        return res.status(400).json({ error: 'Not currently in finish-up timer.' });
    }
    timerRunning = false;
    timeRemaining = 0;
    isPaused = false;
    activeUsage = null;
    console.log('[Server] 2-min finish-up canceled');
    return res.json({ success: true });
});

// Ignore ring => if ringing
app.post('/ignore-ring', (req, res) => {
    if (!ringing) {
        return res.status(400).json({ error: 'Not currently ringing.' });
    }
    timerRunning = false;
    timeRemaining = 0;
    isPaused = false;
    activeUsage = null;
    ringing = false;
    console.log('[Server] Ring ignored => no 2-min started');
    return res.json({ success: true });
});

// Movie => user picks 4
app.post('/use-movie', (req, res) => {
    const { chosenPoints } = req.body;
    if (!Array.isArray(chosenPoints) || chosenPoints.length !== 4) {
        return res.status(400).json({ error: 'Must pick exactly 4 points' });
    }
    for (const p of chosenPoints) {
        let arr = p.category === 'solo' ? soloUsed : p.category === 'together' ? togetherUsed : null;
        if (!arr) return res.status(400).json({ error: 'Invalid category in chosen points.' });
        if (arr[p.index] === true) {
            return res.status(400).json({ error: 'A chosen point is already used.' });
        }
    }
    // Mark used
    for (const p of chosenPoints) {
        let arr = p.category === 'solo' ? soloUsed : togetherUsed;
        arr[p.index] = true;
    }
    const usedAt = new Date().toISOString();
    logs.push({ type: 'movie', index: null, usedAt });
    console.log('[Server] Movie => used 4 chosen points at', usedAt);
    return res.json({ success: true });
});

// Episode => user picks 2
app.post('/use-episode', (req, res) => {
    const { chosenPoints } = req.body;
    if (!Array.isArray(chosenPoints) || chosenPoints.length !== 2) {
        return res.status(400).json({ error: 'Must pick exactly 2 points' });
    }
    for (const p of chosenPoints) {
        let arr = p.category === 'solo' ? soloUsed : p.category === 'together' ? togetherUsed : null;
        if (!arr) return res.status(400).json({ error: 'Invalid category in chosen points.' });
        if (arr[p.index] === true) {
            return res.status(400).json({ error: 'A chosen point is already used.' });
        }
    }
    for (const p of chosenPoints) {
        let arr = p.category === 'solo' ? soloUsed : togetherUsed;
        arr[p.index] = true;
    }
    const usedAt = new Date().toISOString();
    logs.push({ type: 'episode', index: null, usedAt });
    console.log('[Server] Episode => used 2 chosen points at', usedAt);
    return res.json({ success: true });
});

// ----- ADMIN -----
app.post('/admin/login', (req, res) => {
    const { password } = req.body;
    if (password !== ADMIN_PASSWORD) {
        return res.status(403).json({ error: 'Wrong admin password' });
    }
    return res.json({ success: true });
});
app.post('/admin/clear-logs', (req, res) => {
    logs = [];
    console.log('[Server] Logs cleared by admin');
    res.json({ success: true });
});
app.post('/admin/reset-usage', (req, res) => {
    // Keep array lengths, set all to false
    soloUsed = soloUsed.map(() => false);
    togetherUsed = togetherUsed.map(() => false);
    console.log('[Server] Usage reset by admin => all set false, arrays kept');
    res.json({ success: true });
});

// This new route removes all appended points from both arrays, leaving the base 5
app.post('/admin/remove-all-earned', (req, res) => {
    // Keep only the first 5 in each array
    soloUsed.splice(5);
    togetherUsed.splice(5);
    console.log('[Server] All earned points removed => each array back to length 5');
    res.json({ success: true });
});

// For demonstration, these add/remove 1 from "soloUsed"
app.post('/admin/add-earned', (req, res) => {
    soloUsed.push(false);
    console.log('[Server] Admin +1 to soloUsed => length now', soloUsed.length);
    res.json({ success: true });
});
app.post('/admin/remove-earned', (req, res) => {
    if (soloUsed.length <= 0) {
        return res.status(400).json({ error: 'No bubble to remove in soloUsed' });
    }
    soloUsed.pop();
    console.log('[Server] Admin removed last from soloUsed => length now', soloUsed.length);
    res.json({ success: true });
});

app.post('/admin/clear-timer', (req, res) => {
    timerRunning = false;
    timeRemaining = 0;
    isPaused = false;
    activeUsage = null;
    ringing = false;
    console.log('[Server] Timer cleared by admin');
    res.json({ success: true });
});
app.post('/admin/expire-timer', (req, res) => {
    if (timeRemaining > 1) {
        timeRemaining = 1;
        console.log('[Server] Timer => set to 1s => about to end');
    }
    res.json({ success: true });
});

// Start server
app.listen(PORT, () => {
    console.log(`[Server] Listening on port ${PORT}`);
});
