import React, { useState, useEffect, useRef } from 'react';

const SERVER_URL = 'http://localhost:4000';

export default function App() {
  // -----------------------------
  //          SERVER STATE
  // -----------------------------
  const [soloUsed, setSoloUsed] = useState([]);
  const [togetherUsed, setTogetherUsed] = useState([]);
  const [logs, setLogs] = useState([]);

  const [timerRunning, setTimerRunning] = useState(false);
  const [timeRemaining, setTimeRemaining] = useState(0);
  const [isPaused, setIsPaused] = useState(false);
  const [ringing, setRinging] = useState(false);

  // Admin
  const [adminMode, setAdminMode] = useState(false);

  // For status/error messages
  const [statusMessage, setStatusMessage] = useState('');

  // For Movie/Episode selection
  const [chooseMovieMode, setChooseMovieMode] = useState(false);
  const [chooseEpisodeMode, setChooseEpisodeMode] = useState(false);
  const [chosen, setChosen] = useState([]); // e.g. [{ category:'solo', index }, ...]

  // -----------------------------
  //  "ADD POINTS" MULTI-STEP UI
  // -----------------------------
  // Step 1 => enter password + immediate check
  // Step 2 => pick category
  // Step 3 => pick amount
  const [showAddPointsOverlay, setShowAddPointsOverlay] = useState(false);
  const [addPointsStep, setAddPointsStep] = useState(1);
  const [pointsPassword, setPointsPassword] = useState('');
  const [pointsCategory, setPointsCategory] = useState(''); // 'solo' or 'together'
  const [pointsAmount, setPointsAmount] = useState(1);

  // Audio ref for the alarm
  const alarmRef = useRef(null);

  // Poll server
  useEffect(() => {
    fetchState();
    const id = setInterval(fetchState, 1000);
    return () => clearInterval(id);
  }, []);

  // If ringing changes => play/stop audio
  useEffect(() => {
    if (ringing) {
      playAlarm();
    } else {
      stopAlarm();
    }
  }, [ringing]);

  // -----------------------------
  //       FETCH STATE
  // -----------------------------
  async function fetchState() {
    try {
      const res = await fetch(`${SERVER_URL}/state`);
      const data = await res.json();
      setSoloUsed(data.soloUsed || []);
      setTogetherUsed(data.togetherUsed || []);
      setLogs(data.logs || []);
      setTimerRunning(data.timerRunning);
      setTimeRemaining(data.timeRemaining);
      setIsPaused(data.isPaused);
      setRinging(data.ringing);
    } catch (err) {
      console.error(err);
      setStatusMessage('Error: cannot reach server');
    }
  }

  // ALARM
  function playAlarm() {
    if (alarmRef.current) {
      alarmRef.current.loop = true;
      alarmRef.current.currentTime = 0;
      alarmRef.current.play().catch(e => {
        console.log('Alarm might be blocked until user interacts:', e);
      });
    }
  }
  function stopAlarm() {
    if (alarmRef.current) {
      alarmRef.current.pause();
      alarmRef.current.currentTime = 0;
      alarmRef.current.loop = false;
    }
  }

  // SINGLE BUBBLE => 15 MIN
  async function handleBubbleClick(category, index) {
    try {
      const res = await fetch(`${SERVER_URL}/use-bubble`, {
        method: 'POST',
        headers: { 'Content-Type': 'application/json' },
        body: JSON.stringify({ category, index }),
      });
      const data = await res.json();
      if (data.error) {
        setStatusMessage(`Error: ${data.error}`);
      } else {
        setStatusMessage(`Used 1 ${category} (#${index+1}) => 15-min timer`);
      }
    } catch(err) {
      console.error(err);
      setStatusMessage('Error using bubble');
    }
  }

  // TIMER
  async function handlePauseResume() {
    try {
      const res=await fetch(`${SERVER_URL}/pause-resume`,{ method:'POST' });
      const data=await res.json();
      if(data.error) setStatusMessage(`Error: ${data.error}`);
      else setStatusMessage(`Timer paused => ${data.isPaused}`);
    } catch(err){
      console.error(err);
      setStatusMessage('Error pause/resume');
    }
  }
  async function handleFinishUp() {
    try {
      const res=await fetch(`${SERVER_URL}/finish-up`,{method:'POST'});
      const data=await res.json();
      if(data.error) setStatusMessage(`Error: ${data.error}`);
      else setStatusMessage('2-min finish-up started');
    } catch(err){
      console.error(err);
      setStatusMessage('Error finish-up');
    }
  }
  async function handleCancelFinishUp(){
    try {
      const res=await fetch(`${SERVER_URL}/cancel-finish-up`,{ method:'POST'});
      const data=await res.json();
      if(data.error) setStatusMessage(`Error: ${data.error}`);
      else setStatusMessage('Finish-up canceled');
    } catch(err){
      console.error(err);
      setStatusMessage('Error cancel finish-up');
    }
  }
  async function handleIgnoreRing(){
    try {
      const res=await fetch(`${SERVER_URL}/ignore-ring`,{ method:'POST'});
      const data=await res.json();
      if(data.error) setStatusMessage(`Error: ${data.error}`);
      else setStatusMessage('Ignored ring (no 2-min started)');
    } catch(err){
      console.error(err);
      setStatusMessage('Error ignoring ring');
    }
  }

  // MOVIE / EPISODE
  function startChooseMovie(){
    setChooseMovieMode(true);
    setChooseEpisodeMode(false);
    setChosen([]);
    setStatusMessage('Select exactly 4 points for a movie');
  }
  function startChooseEpisode(){
    setChooseEpisodeMode(true);
    setChooseMovieMode(false);
    setChosen([]);
    setStatusMessage('Select exactly 2 points for an episode');
  }
  function toggleChosen(category, index){
    const found=chosen.find(c=>c.category===category && c.index===index);
    if(found){
      setChosen(chosen.filter(c=> c!==found));
    } else {
      setChosen([...chosen, { category,index }]);
    }
  }
  async function submitMovie(){
    if(chosen.length!==4){
      setStatusMessage('Must pick exactly 4 points for a movie');
      return;
    }
    try {
      const res=await fetch(`${SERVER_URL}/use-movie`,{
        method:'POST',
        headers:{'Content-Type':'application/json'},
        body:JSON.stringify({ chosenPoints: chosen }),
      });
      const data=await res.json();
      if(data.error) setStatusMessage(`Error: ${data.error}`);
      else setStatusMessage('Movie => used chosen 4 points');
    } catch(err){
      console.error(err);
      setStatusMessage('Error submitting movie');
    }
    setChooseMovieMode(false);
    setChosen([]);
  }
  async function submitEpisode(){
    if(chosen.length!==2){
      setStatusMessage('Must pick exactly 2 points for an episode');
      return;
    }
    try {
      const res=await fetch(`${SERVER_URL}/use-episode`,{
        method:'POST',
        headers:{'Content-Type':'application/json'},
        body:JSON.stringify({ chosenPoints: chosen }),
      });
      const data=await res.json();
      if(data.error) setStatusMessage(`Error: ${data.error}`);
      else setStatusMessage('Episode => used chosen 2 points');
    } catch(err){
      console.error(err);
      setStatusMessage('Error submitting episode');
    }
    setChooseEpisodeMode(false);
    setChosen([]);
  }

  // ADD POINTS => 3 step flow
  const [passwordChecked, setPasswordChecked] = useState(false); // if true => step2
  function openAddPointsFlow(){
    setShowAddPointsOverlay(true);
    setAddPointsStep(1);
    setPointsPassword('');
    setPointsCategory('');
    setPointsAmount(1);
    setPasswordChecked(false);
  }
  function closeAddPointsFlow(){
    setShowAddPointsOverlay(false);
  }
  // Step 1: user enters password => we check immediately
  async function checkPointsPassword() {
    if(!pointsPassword){
      setStatusMessage('Please enter a password');
      return;
    }
    try {
      const res=await fetch(`${SERVER_URL}/check-password`,{
        method:'POST',
        headers:{'Content-Type':'application/json'},
        body: JSON.stringify({ password: pointsPassword }),
      });
      const data=await res.json();
      if(data.error){
        setStatusMessage(`Error: ${data.error}`);
      } else {
        // password correct => go to step2
        setPasswordChecked(true);
        setAddPointsStep(2);
      }
    } catch(err){
      console.error(err);
      setStatusMessage('Error checking password');
    }
  }
  // Step 2 => pick category
  function pickAddPointsCategory(cat){
    setPointsCategory(cat);
    setAddPointsStep(3);
  }
  // Step 3 => pick amount => do /add-points
  async function pickAddPointsAmount(amt){
    setPointsAmount(amt);
    // Now call POST /add-points
    try {
      const res=await fetch(`${SERVER_URL}/add-points`,{
        method:'POST',
        headers:{'Content-Type':'application/json'},
        body: JSON.stringify({
          password: pointsPassword,
          category: pointsCategory,
          amount: amt
        })
      });
      const data=await res.json();
      if(data.error) setStatusMessage(`Error: ${data.error}`);
      else setStatusMessage(`+${amt} points added to ${pointsCategory}`);
    } catch(err){
      console.error(err);
      setStatusMessage('Error adding points');
    }
    setShowAddPointsOverlay(false);
  }

  // ADMIN
  async function handleAdminLogin(){
    const pw=prompt('Enter admin password:');
    if(!pw) return;
    try {
      const res=await fetch(`${SERVER_URL}/admin/login`,{
        method:'POST',
        headers:{'Content-Type':'application/json'},
        body: JSON.stringify({ password:pw }),
      });
      const data=await res.json();
      if(data.error) setStatusMessage(`Error: ${data.error}`);
      else {
        setAdminMode(true);
        setStatusMessage('Admin mode enabled');
      }
    } catch(err){
      console.error(err);
      setStatusMessage('Error admin login');
    }
  }
  function handleAdminLogout(){
    setAdminMode(false);
    setStatusMessage('Admin mode disabled');
  }
  async function adminRequest(path){
    try {
      const res=await fetch(`${SERVER_URL}/admin/${path}`,{ method:'POST'});
      const data=await res.json();
      if(data.error) setStatusMessage(`Error: ${data.error}`);
      else setStatusMessage(`Admin => ${path} success`);
    } catch(err){
      console.error(err);
      setStatusMessage(`Error admin ${path}`);
    }
  }

  // RENDER BUBBLES
  function renderBubbles(usedArr, category){
    return usedArr.map((val, idx)=>{
      // If chosen => highlight
      const isChosen = chosen.find(c=> c.category===category && c.index===idx);
      const circleColor = val ? '#8B0000' : '#4caf50'; // used => dark red, unused => green

      // Outline color for base vs appended
      let outline = (idx<5) ? '2px solid #5f7b99' : '2px solid #7f5f99';
      if(isChosen) outline='3px solid yellow';

      return (
          <div
              key={idx}
              style={{
                width:30, height:30,
                borderRadius:'50%',
                backgroundColor: circleColor,
                margin:5,
                cursor:'pointer',
                border: outline,
                boxSizing:'border-box'
              }}
              onClick={()=>{
                if(val){
                  setStatusMessage('That bubble is already used.');
                  return;
                }
                if(chooseMovieMode||chooseEpisodeMode){
                  toggleChosen(category,idx);
                } else {
                  handleBubbleClick(category,idx);
                }
              }}
          />
      );
    });
  }

  // Are we in 2-min finish-up?
  const inFinishUp = timerRunning && !ringing && timeRemaining>0 && timeRemaining<=120000;

  // Time formatting
  function formatTime(ms){
    if(ms<=0) return '00:00';
    const totalSec=Math.floor(ms/1000);
    const mm=Math.floor(totalSec/60);
    const ss=totalSec%60;
    return `${String(mm).padStart(2,'0')}:${String(ss).padStart(2,'0')}`;
  }
  function formatLogDate(iso){
    const d=new Date(iso);
    const w=d.toLocaleDateString('en-US',{ weekday:'long'});
    const day=d.getDate();
    const ord=getOrdinal(day);
    const time=d.toLocaleTimeString('en-US',{ hour12:true,hour:'numeric',minute:'2-digit'});
    return `${w} ${day}${ord}, ${time}`;
  }
  function getOrdinal(n){
    const s=['th','st','nd','rd'];
    const v=n%100;
    return s[(v-20)%10]||s[v]||s[0];
  }

  // RENDER
  return (
      <div style={styles.container}>
        <h1>Screen-Time App (Remove All Earned + Immediate PW Check)</h1>

        {statusMessage && <div style={styles.statusBox}>{statusMessage}</div>}

        <audio ref={alarmRef} src="/alarm.mp3" />

        {/* SOLO */}
        <section>
          <h2>Solo Points</h2>
          <div style={styles.bubbleRow}>
            {renderBubbles(soloUsed,'solo')}
          </div>
        </section>

        {/* TOGETHER */}
        <section>
          <h2>Together Points</h2>
          <div style={styles.bubbleRow}>
            {renderBubbles(togetherUsed,'together')}
          </div>
        </section>

        {/* Add Points button => 3-step overlay */}
        <button style={styles.button} onClick={openAddPointsFlow}>
          Add Points
        </button>

        {/* MOVIE / EPISODE */}
        <section style={{marginTop:20}}>
          {chooseMovieMode?(
              <div>
                <p>Select exactly 4 points => Movie</p>
                <button style={styles.button} onClick={submitMovie}>Submit Movie</button>
                <button style={styles.button} onClick={()=>{
                  setChooseMovieMode(false);
                  setChosen([]);
                }}>Cancel</button>
              </div>
          ):(
              <button style={styles.button} onClick={startChooseMovie}>Movie (4 points)</button>
          )}
          {chooseEpisodeMode?(
              <div>
                <p>Select exactly 2 points => Episode</p>
                <button style={styles.button} onClick={submitEpisode}>Submit Episode</button>
                <button style={styles.button} onClick={()=>{
                  setChooseEpisodeMode(false);
                  setChosen([]);
                }}>Cancel</button>
              </div>
          ):(
              <button style={styles.button} onClick={startChooseEpisode}>Episode (2 points)</button>
          )}
        </section>

        {/* Timer */}
        <section style={{ marginTop:30 }}>
          {(timerRunning || timeRemaining>0) && (
              <div>
                <h3>Timer: {formatTime(timeRemaining)} {isPaused && '(Paused)'}</h3>
                <button style={styles.button} onClick={handlePauseResume}>
                  {isPaused? 'Resume' : 'Pause'}
                </button>
              </div>
          )}
          {ringing && (
              <div style={{ marginTop:10 }}>
                <p style={{ color:'yellow' }}>Timer ended! Alarm is ringing!</p>
                <button style={styles.button} onClick={handleFinishUp}>
                  Finish Up (2 min)
                </button>
                <button style={styles.button} onClick={handleIgnoreRing}>
                  Ignore
                </button>
              </div>
          )}
          {inFinishUp && (
              <div style={{ marginTop:10 }}>
                <button style={styles.button} onClick={handleCancelFinishUp}>
                  Cancel Finish Up
                </button>
              </div>
          )}
        </section>

        {/* Logs */}
        <section style={{ marginTop:20 }}>
          <h3>Recent Logs (last 5)</h3>
          {logs.slice(-5).reverse().map((entry,i)=>(
              <p key={i}>
                [{entry.type.toUpperCase()}]
                {entry.index!=null && ` #${entry.index+1}`}
                {' '}at {formatLogDate(entry.usedAt)}
              </p>
          ))}
        </section>

        {/* ADD POINTS OVERLAY */}
        {showAddPointsOverlay && (
            <div style={styles.overlay}>
              <div style={styles.addPointsContainer}>
                {addPointsStep === 1 && (
                    <>
                      <h3>Step 1: Enter Password</h3>
                      <input
                          style={styles.input}
                          type="password"
                          value={pointsPassword}
                          onChange={e=> setPointsPassword(e.target.value)}
                      />
                      <button style={styles.button} onClick={checkPointsPassword}>
                        Check Password
                      </button>
                      <button style={{...styles.button, backgroundColor:'#999'}} onClick={closeAddPointsFlow}>
                        Cancel
                      </button>
                    </>
                )}

                {addPointsStep === 2 && (
                    <>
                      <h3>Step 2: Choose Category</h3>
                      <div style={styles.buttonRow}>
                        <button style={styles.button} onClick={()=> pickAddPointsCategory('solo')}>
                          Solo
                        </button>
                        <button style={styles.button} onClick={()=> pickAddPointsCategory('together')}>
                          Together
                        </button>
                      </div>
                      <button style={{...styles.button, backgroundColor:'#999'}} onClick={closeAddPointsFlow}>
                        Cancel
                      </button>
                    </>
                )}

                {addPointsStep === 3 && (
                    <>
                      <h3>Step 3: How many points?</h3>
                      <div style={styles.buttonRow}>
                        {[1,2,3,4].map(amt=>(
                            <button
                                key={amt}
                                style={styles.button}
                                onClick={()=> pickAddPointsAmount(amt)}
                            >
                              +{amt}
                            </button>
                        ))}
                      </div>
                      <button style={{...styles.button, backgroundColor:'#999'}} onClick={closeAddPointsFlow}>
                        Cancel
                      </button>
                    </>
                )}
              </div>
            </div>
        )}

        {/* Admin Panel */}
        {adminMode?(
            <div style={styles.adminPanel}>
              <h2>Admin Panel</h2>
              <button style={styles.adminButton} onClick={()=>adminRequest('clear-logs')}>Clear Logs</button>
              <button style={styles.adminButton} onClick={()=>adminRequest('reset-usage')}>Reset Usage</button>
              {/* The new remove-all-earned points button */}
              <button style={styles.adminButton} onClick={()=>adminRequest('remove-all-earned')}>
                Remove All Earned
              </button>
              <button style={styles.adminButton} onClick={()=>adminRequest('add-earned')}>+1 (SoloUsed)</button>
              <button style={styles.adminButton} onClick={()=>adminRequest('remove-earned')}>-1 (SoloUsed)</button>
              <button style={styles.adminButton} onClick={()=>adminRequest('clear-timer')}>Clear Timer</button>
              <button style={styles.adminButton} onClick={()=>adminRequest('expire-timer')}>Expire Timer (1s)</button>
              <button style={styles.adminButton} onClick={handleAdminLogout}>Logout Admin</button>
            </div>
        ):(
            <div style={{marginTop:30}}>
              <button style={styles.adminButton} onClick={handleAdminLogin}>
                Admin Login
              </button>
            </div>
        )}
      </div>
  );
}

// -----------------------------
//           STYLES
// -----------------------------
const styles = {
  container: {
    maxWidth:800,
    margin:'20px auto',
    fontFamily:'sans-serif',
    backgroundColor:'#222',
    color:'#fff',
    padding:20,
    borderRadius:6
  },
  statusBox: {
    backgroundColor:'#333',
    padding:10,
    marginBottom:10,
    borderRadius:4
  },
  bubbleRow: {
    display:'flex',
    flexWrap:'wrap',
    justifyContent:'center',
    marginTop:10,
    marginBottom:10
  },
  button: {
    backgroundColor:'#4caf50',
    border:'none',
    color:'#fff',
    padding:'8px 12px',
    margin:5,
    borderRadius:4,
    cursor:'pointer'
  },
  overlay: {
    position:'fixed',
    top:0, left:0, right:0, bottom:0,
    backgroundColor:'rgba(0,0,0,0.6)',
    display:'flex',
    justifyContent:'center',
    alignItems:'center',
    zIndex:9999
  },
  addPointsContainer: {
    backgroundColor:'#333',
    padding:20,
    borderRadius:8,
    width:'80%',
    maxWidth:400,
    textAlign:'center'
  },
  input: {
    width:'80%',
    padding:8,
    margin:'10px 0',
    borderRadius:4,
    border:'1px solid #999',
    fontSize:16
  },
  buttonRow: {
    display:'flex',
    justifyContent:'center',
    flexWrap:'wrap',
    marginTop:10,
    marginBottom:10
  },
  adminPanel: {
    marginTop:30,
    backgroundColor:'#333',
    padding:20,
    borderRadius:4
  },
  adminButton: {
    backgroundColor:'#9C27B0',
    border:'none',
    color:'#fff',
    padding:'6px 10px',
    margin:5,
    borderRadius:4,
    cursor:'pointer'
  }
};
